@@ -2,8 +2,7 @@ from __future__ import absolute_import
 
 from thriftworker.workers.threads import ThreadsWorker
 from thriftworker.constants import DEFAULT_ENV
-from thriftworker.tests.utils import TestCase, custom_env_needed, \
-    start_stop_ctx
+from thriftworker.tests.utils import TestCase, custom_env_needed
 
 from .utils import WorkerMixin
 
@@ -13,6 +12,5 @@ class TestThreadsWorker(WorkerMixin, TestCase):
 
     Worker = ThreadsWorker
 
-    def test_start_stop(self):
-        with start_stop_ctx(self.Worker()) as worker:
-            pass
+    def test_request(self):
+        self.check_request(self.Worker())
