@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import socket
 import errno
 import logging
 from contextlib import contextmanager
@@ -10,15 +11,16 @@ from pyuv.errno import strerror
 from six import with_metaclass
 
 from thriftworker.constants import BACKLOG_SIZE
-from thriftworker.utils.mixin import LoopMixin
+from thriftworker.utils.mixin import LoopMixin, StartStopMixin
 from thriftworker.utils.loop import in_loop
 from thriftworker.utils.decorators import cached_property
+from thriftworker.utils.waiting import wait
 
 logger = logging.getLogger(__name__)
 
 
 @contextmanager
-def ignore_eagain(socket):
+def ignore_eagain():
     """Ignore all *EAGAIN* errors in context."""
     try:
         yield
@@ -36,6 +38,12 @@ class Connections(object):
     def __len__(self):
         return len(self.connections)
 
+    def __iter__(self):
+        return iter(self.connections)
+
+    def __repr__(self):
+        return repr(self.connections)
+
     def register(self, connection):
         """Register new connection."""
         self.connections.add(connection)
@@ -52,10 +60,13 @@ class Connections(object):
         while connections:
             connection = connections.pop()
             if not connection.is_closed():
+                logger.warn('Connection %r closed in worker shutdown',
+                            connection)
                 connection.close()
 
 
 class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
+    """Accept incoming connections."""
 
     Connections = Connections
 
@@ -72,6 +83,7 @@ class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
 
     @cached_property
     def _socket(self):
+        """Create socket from given descriptor."""
         socket = self.app.env.socket
         sock = socket.fromfd(self.descriptor, socket.AF_INET,
                              socket.SOCK_STREAM)
@@ -96,26 +108,30 @@ class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
         """Is current acceptor active."""
         return self._poller.active
 
-    def create_acceptor(self):
+    @cached_property
+    def acceptor(self):
         """Return function that should accept new connections."""
         loop = self.loop
         service = self.name
         connections = self._connections
-        producer = self.app.worker.create_producer(service)
-        socket = self.app.env.socket
         listen_sock = self._socket
+        worker = self.app.worker
+        producer = worker.create_producer(service)
+        concurrency = worker.concurrency
 
         def on_close(connection):
             """Callback called when connection closed."""
             connections.remove(connection)
 
-        def accept_connection(handle, events, error):
+        def inner_acceptor(handle, events, error):
             """Function that try to accept new connection."""
             if error:  # pragma: no cover
                 logger.error('Error handling new connection for'
                              ' service %r: %s', service, strerror(error))
                 return
-            with ignore_eagain(socket):
+            if concurrency.reached:
+                return
+            with ignore_eagain():
                 sock, addr = listen_sock.accept()
                 # Disable Nagle's algorithm for socket.
                 sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
@@ -125,15 +141,17 @@ class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
                                              on_close)
                 connections.register(connection)
 
-        return accept_connection
+        return inner_acceptor
 
     @in_loop
     def start(self):
+        """Start acceptor if active."""
         if not self.active:
-            self._poller.start(UV_READABLE, self.create_acceptor())
+            self._poller.start(UV_READABLE, self.acceptor)
 
     @in_loop
     def stop(self):
+        """Stop acceptor if active."""
         if self.active:
             self._poller.stop()
 
@@ -145,7 +163,7 @@ class BaseAcceptor(with_metaclass(ABCMeta, LoopMixin)):
         self._socket.close()
 
 
-class Acceptors(LoopMixin):
+class Acceptors(StartStopMixin, LoopMixin):
     """Maintain pool of acceptors. Start them when needed."""
 
     def __init__(self):
@@ -166,18 +184,31 @@ class Acceptors(LoopMixin):
         self._acceptors[name] = self.Acceptor(name, fd, backlog=backlog)
 
     def start_by_name(self, name):
+        """Start acceptor by name."""
         acceptor = self._acceptors[name]
-        self.app.loop_container.callback(acceptor.start)
+        self.app.hub.callback(acceptor.start)
 
     def stop_by_name(self, name):
+        """Stop acceptor by name."""
         acceptor = self._acceptors[name]
-        self.app.loop_container.callback(acceptor.stop)
+        self.app.hub.callback(acceptor.stop)
 
-    @in_loop
-    def start(self):
-        pass
+    def start_accepting(self):
+        """Start all registered acceptors if needed."""
+        for acceptor in self._acceptors.values():
+            acceptor.start()
+
+    def stop_accepting(self):
+        """Stop all registered acceptors if needed."""
+        for acceptor in self._acceptors.values():
+            acceptor.stop()
 
-    @in_loop
     def stop(self):
+        """Close all registered acceptors."""
+        self.stop_accepting()
+        # wait for unclosed connections
+        wait(lambda: all(acceptor.connections_number == 0
+                         for acceptor in self._acceptors.values()),
+             timeout=5.0)
         for acceptor in self._acceptors.values():
             acceptor.close()
